﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace FaceSharp.MVC.Code.Results
{
    public class ImageResult : ActionResult
    {
        public ImageResult()
        {
            this.ForceDownload = false;
        }

        public Image Image { get; set; }
        public ImageFormat ImageFormat { get; set; }

        /// <summary>
        /// Defaults to false.
        /// </summary>
        public bool ForceDownload { get; set; }


        /// <summary>
        /// Required if ForceDownload is true, otherwise defaults to "Image.[filetype]"
        /// </summary>
        public string DownloadFileName { get; set; }

        public override void ExecuteResult(ControllerContext context)
        {
            // verify properties
            if (Image == null)
            {
                throw new ArgumentNullException("Image");
            }
            if (ImageFormat == null)
            {
                throw new ArgumentNullException("ImageFormat");
            }

            // output
            context.HttpContext.Response.Clear();

            if (ImageFormat.Equals(ImageFormat.Bmp)) context.HttpContext.Response.ContentType = "image/bmp";
            if (ImageFormat.Equals(ImageFormat.Gif)) context.HttpContext.Response.ContentType = "image/gif";
            if (ImageFormat.Equals(ImageFormat.Icon)) context.HttpContext.Response.ContentType = "image/vnd.microsoft.icon";
            if (ImageFormat.Equals(ImageFormat.Jpeg)) context.HttpContext.Response.ContentType = "image/jpeg";
            if (ImageFormat.Equals(ImageFormat.Png)) context.HttpContext.Response.ContentType = "image/png";
            if (ImageFormat.Equals(ImageFormat.Tiff)) context.HttpContext.Response.ContentType = "image/tiff";
            if (ImageFormat.Equals(ImageFormat.Wmf)) context.HttpContext.Response.ContentType = "image/wmf";

            if (ForceDownload)
            {
                var filename = DownloadFileName ?? "image." + ImageFormat.ToString();
                context.HttpContext.Response.AddHeader("content-disposition", "attachment; filename=\"" + filename + "\"");
            }

            Image.Save(context.HttpContext.Response.OutputStream, ImageFormat);
        }
    }
}